<?php
/**
 * Classe que permite aos processos workflow fazer consultas ao Organograma
 *
 * @author Sidnei Augusto Drovetto Junior - drovetto@gmail.com
 * @version 1.2
 * @license http://www.gnu.org/copyleft/gpl.html GPL
 * @package Workflow
 * @subpackage local
 */
class wf_orgchart
{
	/**
	 * @var object $db Link para o Banco de Dados do Workflow.
	 * @access private
	 */
	var $db;

	/**
	 * Construtor da classe wf_orgchart
	 * @return object
	 * @access public
	 */
	function wf_orgchart()
	{
		$this->db = &$GLOBALS['workflow']['workflowObjects']->getDBWorkflow()->Link_ID;
		$this->db->setFetchMode(ADODB_FETCH_ASSOC);
	}

	/**
	 * Busca uma organizao pelo seu ID.
	 *
	 * Este mtodo ir procurar uma organizao na tabela de organizaes, pelo seu ID, e retornar seu dados bsicos.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array associativa contendo os atributos de uma organizao:
	 * - organizacao_id
	 * - nome: o nome abreviado da organizao
	 * - descrio: o nome completo da organizao
	 * - url_imagem: a url onde se encontra o grfico da organizao
	 * - ativa: se a organizao est ativa ou no
	 * - sitio: a url da pgina web da organizao
	 * @access public
	 */
	function getOrganization($organizationID)
	{
		$query = "SELECT organizacao_id, nome, descricao, url_imagem, ativa, sitio" .
				 "  FROM organizacao" .
				 " WHERE (organizacao_id = ?)";

		$result = $this->db->query($query, array($organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca uma organizao pelo seu nome.
	 *
	 * Este mtodo ir buscar os dados bsicos de uma organizao, procurando pela sua sigla.
	 * @param string $name A sigla da organizao.
	 * @return array Uma array associativa contendo os atributos de uma organizao:
	 * - organizacao_id
	 * - nome: a sigla da organizao
	 * - descricao: o nome completo
	 * - url_imagem: a url onde se encontra o grfico da organizao
	 * - ativa: se a organizao est ativa ou no
	 * - sitio: a url da pgina web da organizao
	 * @access public
	 */
	function getOrganizationByName($name)
	{
		$query = "SELECT organizacao_id, nome, descricao, url_imagem, ativa, sitio" .
				 "  FROM organizacao" .
				 " WHERE (UPPER(nome) = UPPER(?))";

		$result = $this->db->query($query, array($name));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Lista todos os telefones teis de uma organizao.
	 *
	 * Este mtodo ir listar a tabela telefone.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array de arrays associativas contendo a lista dos telefones de uma organizao:
	 * - telefone_id
	 * - descricao
	 * - organizacao_id
	 * @access public
	 */
	function getOrganizationTelephones($organizationID)
	{
		$query = "SELECT telefone_id, descricao, organizacao_id" .
				 "  FROM telefone" .
				 "	WHERE organizacao_id = ?";

		$result = $this->db->query($query, array((int) $organizationID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Busca as reas de uma organizao
	 *
	 * Este mtodo ir buscar na tabela de reas, todas as reas que pertencem  organizao solicitada.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array seqencial contendo as reas de uma organizao. Cada linha do array conter:
	 * - organizacao_id
	 * - area_id
	 * - area_status_id: corresponde ao nvel hierrquico da area
	 * - superior_area_id: o id da rea acima da atual
	 * - centro_custo_id: o id do centro de custo da area
	 * - titular_funcionario_id: o id do funcionario titular da rea. Corresponde ao uidNumber do funcionrio no catlogo Ldap.
	 * - substituto_funcionario_id: o id do funcionario que est substituindo o titular temporariamente
	 * - sigla: sigla da area
	 * - descrio: nome completo da area
	 * - ativa: indicativo de situao da area, sendo 's' ativa, e 'n' inativa
	 * - auxiliar_funcionario_id: id da secretria da rea
	 * @access public
	 */
	function getOrganizationAreas($organizationID)
	{
		$query = "SELECT organizacao_id, area_id, area_status_id, " .
				 "       superior_area_id, centro_custo_id, titular_funcionario_id," .
				 "       substituto_funcionario_id, sigla, descricao, ativa, auxiliar_funcionario_id" .
				 "  FROM area" .
				 " WHERE (organizacao_id = ?)" .
				 " ORDER BY sigla, descricao";

		$result = $this->db->query($query, array($organizationID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Busca os status de rea de uma organizao.
	 *
	 * O status de rea deve ser compreendido como um nvel hirrquico das reas da organizao.
	 * Por exemplo: presidncia, assessoria, diretoria, gerncia, diviso, etc.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array de arrays associativas contendo os atributos dos status de rea. Cada linha do array conter:
	 * - area_status_id
	 * - organizao_id
	 * - descrio
	 * - nvel: a posio hierrquica do nvel no organograma. Por exemplo: 1 - presidencia, 2 - assessoria, etc
	 * @access public
	 */
	function getOrganizationAreaStatus($organizationID)
	{
		$query = "SELECT area_status_id, organizacao_id, descricao, nivel" .
				 "  FROM area_status" .
				 " WHERE (organizacao_id = ?)";

		$result = $this->db->query($query, array($organizationID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Busca as localidades de uma organizao.
	 *
	 * As localidades de uma organizao representam o local fsico de trabalho dos funcionrios.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array de arrays associativas contendo os atributos das localidades. Cada linha do array conter:
	 * - organizacao_id
	 * - localidade_id
	 * - centro_custo_id
	 * - descricao: o nome simplificado localidade
	 * - empresa: o nome completo da localidade
	 * - endereo: o logradouro da empresa, com o nmero
	 * - complemento: dado adicional do endereo
	 * - cep: cdigo de endereamento postal, mscara nnnnnn-nnn
	 * - bairro: nome do bairro
	 * - cidade: nome da cidade
	 * - uf: unidade da federao
	 * @access public
	 */
	function getOrganizationLocals($organizationID)
	{
		$query = "SELECT organizacao_id, localidade_id, centro_custo_id, descricao, empresa,
							endereco, complemento, cep, bairro, cidade, uf" .
				 "  FROM localidade" .
				 " WHERE (organizacao_id = ?)";

		$result = $this->db->query($query, array($organizationID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Busca os centros de custo de uma organizao.
	 *
	 * Este mtodo retornar todos os centros de custo de uma organizao.
	 * Centros de custo so como cdigos contbeis para faturamento de servios.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array de arrays associativas contendo os atributos dos centros de custo. Cada linha do array conter:
	 * - organizacao_id
	 * - centro_custo_id
	 * - nm_centro_custo: nmero do centro de custo
	 * - grupo: estrutura numerica a qual o centro de custo pertence
	 * - descrio: nome do centro de custo
	 * @access public
	 */
	function getOrganizationCostCenters($organizationID)
	{
		$query = "SELECT organizacao_id, centro_custo_id, nm_centro_custo, grupo, descricao" .
				 "  FROM centro_custo" .
				 " WHERE (organizacao_id = ?)" .
				 " ORDER BY descricao";

		$result = $this->db->query($query, array($organizationID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Lista todas as categorias possveis para um funcionrio em uma organizao.
	 *
	 * Este mtodo listar a tabela de categorias.
	 * Por exemplo: funcionrio, estagirio, terceirizado, etc.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array de arrays associativas contendo os atributos das categorias. Cada linha do array conter:
	 * - funcionario_categoria_id: o id da categoria
	 * - descricao
	 * - organizacao_id
	 * @access public
	 */
	function getOrganizationEmployeeCategories($organizationID)
	{
		$query = "SELECT funcionario_categoria_id, descricao, organizacao_id" .
				 "  FROM funcionario_categoria" .
				 "	WHERE organizacao_id = ?" .
				 "  ORDER BY funcionario_categoria_id";

		$result = $this->db->query($query, array((int) $organizationID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Lista todas as organizaes do Organograma.
	 *
	 * Este mtodo ir listar a tabela de organizaes
	 * O modelo de dados do organograma foi construdo para abrigar mais de uma organizao
	 * @return um array de arrays associativas contendo a lista de organizaes. Cada linha do array conter:
	 * - organizacao_id
	 * - nome: sigla da organizacao
	 * - descricao
	 * - url_imagem: a url onde se encontra o grfico da organizao
	 * - ativa: se a organizao est ativa ou no
	 * - sitio: a url da pgina web da organizao
	 * @access public
	 */
	function getOrganizations()
	{
		$query = "SELECT organizacao_id, nome, descricao, url_imagem, ativa, sitio" .
				 "  FROM organizacao";

		$result = $this->db->query($query);
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}


	/**
	 * Busca uma rea pelo seu ID.
	 *
	 * Este mtodo ir retornar os dados de uma rea buscando pelo seu ID.
	 * @param int $areaID O ID da rea.
	 * @return array Uma array associativa contendo os atributos de uma rea:
	 * - organizacao_id
	 * - area_id
	 * - area_status: o nvel hierrquico da rea
	 * - superior_area_id: o id da rea acima da atual
	 * - centro_custo_id: o centro de custo da rea
	 * - titular_funcionario_id: o id do chefe da rea
	 * - substituto_funcionario_id: o id do funcionario que esta substituindo o titular temporariamente
	 * - sigla
	 * - descicao
	 * - ativa: indicativo de situao da rea: 's' ativa, 'n' inativa
	 * - auxiliar_funcionario_id: o id da secretria da rea
	 * @access public
	 */
	function getArea($areaID)
	{
		$query = "SELECT organizacao_id, area_id, area_status_id, " .
				 "       superior_area_id, centro_custo_id, titular_funcionario_id," .
				 "       substituto_funcionario_id, sigla, descricao, ativa, auxiliar_funcionario_id" .
				 "  FROM area" .
				 " WHERE (area_id = ?)";

		$result = $this->db->query($query, array($areaID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca uma rea pela sua sigla.
	 *
	 * Este mtodo retornar os atributos de uma rea buscando pela sua sigla.
	 * @param string $acronym A sigla da rea.
	 * @param int $organizationID O id da organizao
	 * @return array Uma array associativa contendo os atributos de uma rea:
	 * - organizacao_id
	 * - area_id
	 * - area_status: o nvel hierrquico da rea
	 * - superior_area_id: o id da rea acima da atual
	 * - centro_custo_id: o centro de custo da rea
	 * - titular_funcionario_id: o id do chefe da rea
	 * - substituto_funcionario_id: o id do funcionario que esta substituindo o titular temporariamente
	 * - sigla
	 * - descicao
	 * - ativa: indicativo de situao da rea: 's' ativa, 'n' inativa
	 * - auxiliar_funcionario_id: o id da secretria da rea
	 * @access public
	 */
	function getAreaByName($acronym, $organizationID = 1)
	{
		$query = "SELECT organizacao_id, area_id, area_status_id, " .
				 "       superior_area_id, centro_custo_id, titular_funcionario_id," .
				 "       substituto_funcionario_id, sigla, descricao, ativa, auxiliar_funcionario_id" .
				 "  FROM area" .
				 " WHERE (UPPER(sigla) = UPPER(?) and (organizacao_id = ?))";

		$result = $this->db->query($query, array($acronym, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca as reas que possuem um determinado status de rea.
	 *
	 * Este mtodo ir retornar todas as reas cujo status (nvel hierrquico) seja o solicitado.
	 * @param int $areaStatusID O ID do status de rea.
	 * @return array Uma array de arrays associativas contendo os atributos de uma rea. Cada linha do array conter:
	 * - organizacao_id
	 * - area_id
	 * - area_status: o nvel hierrquico da rea
	 * - superior_area_id: o id da rea acima da atual
	 * - centro_custo_id: o centro de custo da rea
	 * - titular_funcionario_id: o id do chefe da rea
	 * - substituto_funcionario_id: o id do funcionario que esta substituindo o titular temporariamente
	 * - sigla
	 * - descicao
	 * - ativa: indicativo de situao da rea: 's' ativa, 'n' inativa
	 * - auxiliar_funcionario_id: o id da secretria da rea
	 * @access public
	 */
	function getAreaByStatus($areaStatusID)
	{
		$result = array();

		$query = "SELECT organizacao_id, area_id, area_status_id, " .
				 "       superior_area_id, centro_custo_id, titular_funcionario_id," .
				 "       substituto_funcionario_id, sigla, descricao, ativa, auxiliar_funcionario_id" .
				 "  FROM area" .
				 " WHERE (area_status_id = ?)";

		$result = $this->db->query($query, array($areaStatusID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Busca vrias reas atravs de uma array de IDs
	 *
	 * Este mtodo ir buscar de uma vez s os dados de mais de uma rea.
	 * @param array $areaIDs Array com os IDs das reas
	 * @return array Um array de arrays associativos contendo os atributos de vrias reas. Cada linha do array conter:
	 * - organizacao_id
	 * - area_id
	 * - area_status: o nvel hierrquico da rea
	 * - superior_area_id: o id da rea acima da atual
	 * - centro_custo_id: o centro de custo da rea
	 * - titular_funcionario_id: o id do chefe da rea
	 * - substituto_funcionario_id: o id do funcionario que esta substituindo o titular temporariamente
	 * - sigla
	 * - descicao
	 * - ativa: indicativo de situao da rea: 's' ativa, 'n' inativa
	 * - auxiliar_funcionario_id: o id da secretria da rea
 	 * @access public
	 */
	function getAreas($areaIDs)
	{
		if (!is_array($areaIDs))
			return false;

		$areas = implode(', ', $areaIDs);

		$query = "SELECT organizacao_id, area_id, area_status_id, " .
				 "       superior_area_id, centro_custo_id, titular_funcionario_id," .
				 "       substituto_funcionario_id, sigla, descricao, ativa, auxiliar_funcionario_id" .
				 "  FROM area" .
				 " WHERE area_id IN ($areas)" .
				 " ORDER BY sigla, descricao";

		// A execuo  realizada sem o segundo parmetro pois este no pode estar entre aspas
		$result = $this->db->query($query);
		if (!$result)
			return false;

		$output = $result->GetArray(-1);
		return $output;
	}

	/**
	 * Busca o ID da rea superior a uma dada rea.
	 *
	 * Este mtodo ir buscar a rea imediatamente superior  solicitada, ou ento subir na hierarquia at chegar no nvel solicitado.
	 * @param int $areaID O ID da rea da qual se quer saber a rea superior.
	 * @param int $areaStatusID O ID do status de rea (nvel) da rea pai. Utilizar -1 caso se queira a rea imediatamente superior.
	 * - Por exemplo: rea atual est no nvel 5 (Diviso) e se quer buscar a rea de nivel 3 (Diretoria).
	 * @return int O ID da rea que  superior  rea informada.
	 * @access public
	 */
	function getParentAreaID($areaID, $areaStatusID = -1)
	{
		$query  = "SELECT area_id, area_status_id" .
				  "  FROM area" .
				  " WHERE area_id = (SELECT superior_area_id" .
				  "                    FROM area" .
				  "                   WHERE (area_id = ?))";

		$result = $this->db->query($query, array($areaID));
		if (!$result)
			return false;

		$output = $result->fetchRow();
		if (!$output)
			return false;

		if (($areaStatusID == -1) || ($output['area_status_id'] == $areaStatusID))
			return $output['area_id'];
		else
			return $this->getParentAreaID($output['area_id'], $areaStatusID);
	}

	/**
	 * Busca as reas abaixo de uma determinada rea.
	 *
	 * Este mtodo ir buscar as reas imediatamente inferiores  solicitada, no descendo nos prximos nveis da hierarquia.
	 * @param int $parentAreaID O ID da rea da qual se quer saber as reas imediatamente inferiores.
	 * @param boolean $onlyActiveAreas Valor lgico que, caso verdadeiro, faz com que o mtodo retorne somente as reas ativas
	 * @return array Um array de arrays associativos contendo os atributos de vrias reas. Cada linha do array conter:
	 * - organizacao_id
	 * - area_id
	 * - area_status: o nvel hierrquico da rea
	 * - superior_area_id: o id da rea acima da atual
	 * - centro_custo_id: o centro de custo da rea
	 * - titular_funcionario_id: o id do chefe da rea
	 * - substituto_funcionario_id: o id do funcionario que esta substituindo o titular temporariamente
	 * - sigla
	 * - descicao
	 * - ativa: indicativo de situao da rea: 's' ativa, 'n' inativa
	 * - auxiliar_funcionario_id: o id da secretria da rea
	 * @access public
	 */
	function getSubAreasByParentAreaID($parentAreaID, $onlyActiveAreas = false)
	{
		$query = "SELECT organizacao_id, area_id, area_status_id, " .
				 "       superior_area_id, centro_custo_id, titular_funcionario_id," .
				 "       substituto_funcionario_id, sigla, descricao, ativa, auxiliar_funcionario_id" .
				 "  FROM area" .
				 " WHERE superior_area_id = ?";

		if ($onlyActiveAreas){
			$query .= " AND ativa = 'S'";
		}
		$query .= " ORDER BY sigla, descricao";

		$result = $this->db->query($query, array($parentAreaID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);
		return $output;
	}

	/**
	 * Busca um status de rea pelo seu ID.
	 *
	 * Procura na tabela de status de rea (nvel hierrquico de reas) o registro que corresponde ao ID solicitado.
	 * @param int $areaStatusID O ID do status de rea.
	 * @return array Uma array associativa contendo os atributos de um status de rea:
	 * - area_status_id
	 * - organizacao_id
	 * - descricao: nome do nvel hierrquivo. Por exemplo: presidncia, assessoria, diretoria, gerncia, etc.
	 * - nivel: valor numrico que identifica o nvel: 1, 2, 3, ...
	 * @access public
	 */
	function getAreaStatus($areaStatusID)
	{
		$query = "SELECT area_status_id, organizacao_id, descricao, nivel" .
				 "  FROM area_status" .
				 " WHERE (area_status_id = ?)";

		$result = $this->db->query($query, array($areaStatusID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca um status de rea pelo seu nome.
	 *
	 * Este mtodo ir retornar os dados de um status, procurando o registro na tabela atravs do seu nome.
	 * @param string $description O nome do status de rea.
	 * @param int $organizationID O id da organizao.
	 * @return array Uma array associativa contendo os atributos de um status de rea:
	 * - area_status_id
	 * - organizacao_id
	 * - descricao: nome do nvel hierrquivo. Por exemplo: presidncia, assessoria, diretoria, gerncia, etc.
	 * - nivel: valor numrico que identifica o nvel: 1, 2, 3, ...
	 * @access public
	 */
	function getAreaStatusByName($description, $organizationID = 1)
	{
		$query = "SELECT area_status_id, organizacao_id, descricao, nivel" .
				 "  FROM area_status" .
				 " WHERE (UPPER(descricao) = UPPER(?) and (organizacao_id = ?))";

		$result = $this->db->query($query, array($description, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca um status de rea pelo seu nvel.
	 *
	 * Este mtodo ir retornar os dados de um status, procurando o registro na tabela atravs do seu nvel.
	 * @param int $level O nvel do status de rea.
	 * @param int $organizationID O id da organizao.
	 * @return array Uma array associativa contendo os atributos de um status de rea:
	 * - area_status_id
	 * - organizacao_id
	 * - descricao: nome do nvel hierrquivo. Por exemplo: presidncia, assessoria, diretoria, gerncia, etc.
	 * - nivel: valor numrico que identifica o nvel: 1, 2, 3, ...
	 * @access public
	 */
	function getAreaStatusByLevel($level, $organizationID = 1)
	{
		$query = "SELECT area_status_id, organizacao_id, descricao, nivel" .
				 " FROM area_status" .
				 " WHERE (nivel = ?) AND (organizacao_id = ?)";

		$result = $this->db->query($query, array((int) $level, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca o ID do titular de uma rea.
	 *
	 * Este mtodo busca uma rea e retorna o atributo titular_funcionario_id
	 * @param int $areaID O ID da rea.
	 * @return int O ID do titular da rea.
	 * @access public
	 */
	function getAreaSupervisorID($areaID)
	{
		$area = $this->getArea($areaID);
		if (!$area)
			return false;

		return $area['titular_funcionario_id'];
	}

	/**
	 * Busca o ID do substituto de uma rea.
	 *
	 * Este mtodo ir buscar uma rea e retornar o atributo substituto_funcionario_id.
	 * Note que o substituro  um campo opcional na rea e poder retornar vazio.
	 * @param int $areaID O ID da rea.
	 * @return int O ID do substituto da rea.
	 * @access public
	 */
	function getAreaBackupSupervisorID($areaID)
	{
		$area = $this->getArea($areaID);
		if (!$area)
			return false;

		return $area['substituto_funcionario_id'];
	}

	/**
	 * Busca o ID do auxiliar administrativo de uma rea.
	 *
	 * Este mtodo busca uma rea e retorna o atributo auxiliar_funcionario_id
	 * Nem todas as reas possuem funcionrios auxiliares (secretrias)
	 * @param int $areaID O ID da rea.
	 * @return int O ID do auxiliar administrativo da rea.
	 * @access public
	 */
	function getAreaAssistantID($areaID)
	{
		$area = $this->getArea($areaID);
		if (!$area)
			return false;

		return $area['auxiliar_funcionario_id'];
	}

	/**
	 * Busca o ID do titular atual de uma rea.
	 *
	 * Este mtodo ir buscar uma rea e caso haja um substituto, este ser o titular atual; caso contrrio, o titular atual  o prprio titular da rea.
	 * @param int $areaID O ID da rea.
	 * @return int O ID do titular atual da rea, podendo ser um dos campos abaixo:
	 * - substituto_funcionario_id
	 * - titular_funcionario_id
	 * @access public
	 */
	function getAreaCurrentSupervisorID($areaID)
	{
		$area = $this->getArea($areaID);
		if (!$area)
			return false;

		return is_null($area['substituto_funcionario_id']) ? $area['titular_funcionario_id'] : $area['substituto_funcionario_id'];
	}

	/**
	 * Busca o ID do responsvel administrativo atual de uma rea.
	 *
	 * A precedncia para definir quem  o responsvel administrativo : auxiliar administrativo, substituto e titular.
	 * @param int $areaID O ID da rea.
	 * @return int O ID do responsvel administrativo da rea, podendo ser um dos atributos abaixo:
	 * - auxiliar_funcionario_id
	 * - substituto_funcionario_id
	 * - titular_funcionario_id
	 * @access public
	 */
	function getAreaCurrentAdministrativeResponsibleID($areaID)
	{
		$area = $this->getArea($areaID);
		if (!$area)
			return false;

		return !is_null($area['auxiliar_funcionario_id']) ? $area['auxiliar_funcionario_id'] : (!is_null($area['substituto_funcionario_id']) ? $area['substituto_funcionario_id'] : $area['titular_funcionario_id']);
	}

	/**
	 * Busca uma localidade pelo seu ID.
	 *
	 * Este mtodo listar os atributos e uma localidade. Lembrando que localidade tambm pode ser entendida como o local fsico de trabalho.
	 * @param int $localID O ID da localidade.
	 * @return array Uma array associativa contendo os atributos de uma localidade:
	 * - organizacao_id
	 * - localidade_id
	 * - centro_custo_id
	 * - descricao
	 * - empresa: o nome completo da localidade
	 * - endereo: o logradouro da empresa, com o nmero
	 * - complemento: dado adicional do endereo
	 * - cep: cdigo de endereamento postal, mscara nnnnnn-nnn
	 * - bairro: nome do bairro
	 * - cidade: nome da cidade
	 * - uf: unidade da federao
	 * @access public
	 */
	function getLocal($localID)
	{
		$query = "SELECT organizacao_id, localidade_id, centro_custo_id, descricao,
							empresa, endereco, complemento, cep, bairro, cidade, uf" .
				 "  FROM localidade" .
				 " WHERE (localidade_id = ?)";

		$result = $this->db->query($query, array($localID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca uma localidade pelo seu nome.
	 *
	 *
	 * @param string $description O nome da localidade.
	 * @param int $organizationID O id da organizao.
	 * @return array Uma array associativa contendo os atributos de uma localidade:
	 * - organizacao_id
	 * - localidade_id
	 * - centro_custo_id
	 * - descricao
	 * - empresa: o nome completo da localidade
	 * - endereo: o logradouro da empresa, com o nmero
	 * - complemento: dado adicional do endereo
	 * - cep: cdigo de endereamento postal, mscara nnnnnn-nnn
	 * - bairro: nome do bairro
	 * - cidade: nome da cidade
	 * - uf: unidade da federao
	 * @access public
	 */
	function getLocalByName($description, $organizationID = 1)
	{
		$query = "SELECT organizacao_id, localidade_id, centro_custo_id, descricao,
							empresa, endereco, complemento, cep, bairro, cidade, uf" .
				 "  FROM localidade" .
				 " WHERE (UPPER(descricao) = UPPER(?) and (organizacao_id = ?))";

		$result = $this->db->query($query, array($description, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca um centro de custo pelo seu ID.
	 *
	 * Este mtodo ir mostrar os atributos de um centro de custo, buscando na tabela pelo seu id.
	 * @param int $costCenterID O ID do centro de custo.
	 * @return array Uma array associativa contendo os atributos de um centro de custo:
	 * - organizacao_id
	 * - centro_custo_id
	 * - nm_centro_custo: nmero do centro de custo
	 * - grupo: estrutura numerica a qual o centro de custo pertence
	 * - descrio: nome do centro de custo
	 * @access public
	 */
	function getCostCenter($costCenterID)
	{
		$query = "SELECT organizacao_id, centro_custo_id, nm_centro_custo, grupo, descricao" .
				 "  FROM centro_custo" .
				 " WHERE (centro_custo_id  = ?)";

		$result = $this->db->query($query, array($costCenterID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca um centro de custo pelo seu nmero.
	 *
	 * Este mtodo retornar os atributos de um centro de custo buscando pelo seu ID.
	 * @param int $number O nmero do centro de custo.
	 * @param ind $organizationID O id da organizao
	 * @return array Uma array associativa contendo os atributos de um centro de custo:
	 * - organizacao_id
	 * - centro_custo_id
	 * - nm_centro_custo: nmero do centro de custo
	 * - grupo: estrutura numerica a qual o centro de custo pertence
	 * - descrio: nome do centro de custo
	 * @access public
	 */
	function getCostCenterByNumber($number, $organizationID = 1)
	{
		$query = "SELECT organizacao_id, centro_custo_id, nm_centro_custo, grupo, descricao" .
				 "  FROM centro_custo" .
				 " WHERE ((nm_centro_custo = ?) and (organizacao_id = ?))";

		$result = $this->db->query($query, array($number, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca um centro de custo pelo seu nome.
	 *
	 * Este mtodo ir retornar os atributos de um centro de custo buscando pelo seu nome.
	 * @param string $description O nome do centro de custo.
	 * @param int $organizationID O id da organizao
	 * @return array Uma array associativa contendo os atributos de um centro de custo.
	 * - organizacao_id
	 * - centro_custo_id
	 * - nm_centro_custo: nmero do centro de custo
	 * - grupo: estrutura numerica a qual o centro de custo pertence
	 * - descrio: nome do centro de custo
	 * 	 * @access public
	 */
	function getCostCenterByName($description, $organizationID = 1)
	{
		$query = "SELECT organizacao_id, centro_custo_id, nm_centro_custo, grupo, descricao" .
				 "  FROM centro_custo" .
				 " WHERE (UPPER(descricao) = UPPER(?) and (organizacao_id = ?))";

		$result = $this->db->query($query, array($description, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca as informaes de categoria a partir do ID.
	 *
	 * Este mtodo ir retornar os atributos de uma linha da tabela de categorias de funcionrios.
	 * @param int $categoryID O ID da categoria.
	 * @return array Uma array associativa contendo os atributos da categoria:
	 * - funcionario_categoria_id
	 * - descricao
	 * - organizacao_id
	 * @access public
	 */
	function getEmployeeCategory($categoryID)
	{
		$query  = "SELECT funcionario_categoria_id, descricao, organizacao_id" .
				  "  FROM funcionario_categoria" .
				  " WHERE (funcionario_categoria_id = ?)";

		$result = $this->db->query($query, array($categoryID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca uma categoria a partir da sua descrio.
	 *
	 * Este mtodo ir mostrar os atributos de uma categoria, buscando pela sua descrio.
	 * @param string $description O nome da categoria.
	 * @param int $organizationID O id da organizao
	 * @return array Uma array associativa contendo os atributos de uma categoria:
	 * - funcionario_categoria_id
	 * - descricao
	 * - organizacao_id
	 * @access public
	 */
	function getEmployeeCategoryByName($description, $organizationID = 1)
	{
		$query = "SELECT funcionario_categoria_id, descricao, organizacao_id" .
				 "  FROM funcionario_categoria" .
				 " WHERE (UPPER(descricao) = UPPER(?) and (organizacao_id = ?))";

		$result = $this->db->query($query, array($description, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Lista todos os cargos de uma organizao.
	 *
	 * Este mtodo ir listar a tabela localidade.
	 * @param int $organizationID O ID da organizao.
	 * @return array Uma array de arrays associativas contendo a lista dos cargos de uma organizao:
	 * - cargo_id
	 * - descricao
	 * - organizacao_id
	 * @access public
	 */
	function getJobTitles($organizationID)
	{
		$query = "SELECT cargo_id, descricao, organizacao_id" .
				 "  FROM cargo" .
				 "	WHERE organizacao_id = ?" .
				 "  ORDER BY cargo_id";

		$result = $this->db->query($query, array((int) $organizationID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Busca as informaes de cargo a partir do ID.
	 *
	 * Este mtodo ir listar os atributos de um cargo, buscando na tabela a partir do seu ID.
	 * @param int $jobTitleID O ID do cargo.
	 * @return array Uma array associativa contendo os atributos do cargo.
	 * - cargo_id
	 * - descricao
	 * - organizacao_id
	 * @access public
	 */
	function getJobTitle($jobTitleID)
	{
		$query  = "SELECT cargo_id, descricao, organizacao_id" .
				  "  FROM cargo" .
				  " WHERE (cargo_id = ?)";

		$result = $this->db->query($query, array($jobTitleID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca um cargo a partir da sua descrio.
	 *
	 * Este mtodo ir mostrar os atributos de um cargo, buscando pela sua descrio.
	 * @param string $description O nome do cargo.
	 * @param int $organizationID O id da organizao.
	 * @return array Uma array associativa contendo os atributos de um cargo:
	 * - cargo_id
	 * - descricao
	 * - organizacao_id
	 * @access public
	 */
	function getJobTitleByName($description, $organizationID = 1)
	{
		$query = "SELECT cargo_id, descricao, organizacao_id" .
				 "  FROM cargo" .
				 " WHERE (UPPER(descricao) = UPPER(?) and (organizacao_id = ?))";

		$result = $this->db->query($query, array($description, (int) $organizationID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca um funcionrio pelo seu ID.
	 *
	 * Este mtodo retorna os atributos de um funcionrio buscando pelo se id.
	 * O ID corresponde ao uidNumber do funcionrio no catlogo LDAP.
	 * Se necessitar de outros atributos como o nome, cpf, email, matrcula,  necessrio fazer uma consulta ao Ldap.
	 * @param int $employeeID O ID do funcionrio.
	 * @return array Uma array associativa contendo os atributos de um funcionrio:
	 * - funcionario_id
	 * - area_id
	 * - localidade_id
	 * - centro_custo_id
	 * - organizacao_id
	 * - funcionario_status_id
	 * - cargo_id
	 * - nivel: o nvel numrico dentro do cargo
	 * - funcionario_categoria_id
	 * - titulo: nome pelo qual o funcionrio e reconhecido na organizao, por exemplo: gerente comercial
	 * @access public
	 */
	function getEmployee($employeeID)
	{
		if (!is_numeric($employeeID))
			return false;

		$query  = "SELECT funcionario_id, area_id, localidade_id, centro_custo_id, organizacao_id,
							funcionario_status_id, cargo_id, nivel, funcionario_categoria_id, titulo" .
				  "  FROM funcionario" .
				  " WHERE (funcionario_id = ?)";

		$result = $this->db->query($query, array($employeeID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca lista de funcionrios de uma rea pelo ID da rea.
	 *
	 * Este mtodo retornar todos os funcionrios de uma rea.
	 * @param int $areaID O ID da rea.
	 * @param boolean $onlyActiveUsers true para retornar somente usurios ativos e false caso contrrio
	 * @return array Uma array sequencial de arrays associativos contendo contendo o ID dos funcionrios de uma rea:
	 * - funcionario_id
	 * @access public
	 */
	function getEmployeesAreaID($areaID, $onlyActiveUsers = false)
	{
		if($onlyActiveUsers){
			$query = "SELECT funcionario_id " .
					 "FROM funcionario " .
					 "INNER JOIN funcionario_status " .
					 "ON (funcionario.funcionario_status_id = funcionario_status.funcionario_status_id) " .
					 "WHERE (area_id = ?) AND (funcionario_status.exibir = 'S')";
		} else {
			$query = "SELECT funcionario_id " .
				     "FROM funcionario " .
					 "WHERE (area_id = ?)";
		}

		$result = $this->db->query($query, array($areaID));
		if (!$result)
			return false;

		$output = $result->GetArray(-1);

		return $output;
	}

	/**
	 * Busca um status de funcionrio pelo seu ID.
	 *
	 * Este mtodo ir retornar os dados de um status de funcionrio, buscando pelo ID solicitado.
	 * @param int $employeeStatusID O ID do status.
	 * @return array Uma array associativa contendo os atributos de um status de funcionrio:
	 * - funcionario_status_id
	 * - descricao
	 * - exibir: indicativo booleado 'S' funcionrio ser listado no organograma, 'N' funcionrio ficar oculto na aba organograma.
	 * @access public
	 */
	function getEmployeeStatus($employeeStatusID)
	{
		$query = "SELECT funcionario_status_id, descricao, exibir " .
				 "  FROM funcionario_status" .
				 " WHERE (funcionario_status_id = ?)";

		$result = $this->db->query($query, array($employeeStatusID));
		if (!$result)
			return false;

		$output = $result->fetchRow();

		return $output;
	}

	/**
	 * Busca o ID da rea (podendo ser de outros nveis) de um funcionrio.
	 *
	 * Este mtodo ir retornar o ID da rea do funcionrio. Far uma busca para cima na hierarquia se for solicitado.
	 * @param int $employeeID O ID do funcionrio do qual se quer saber a rea.
	 * @param int $areaStatusID O ID do status de rea (nvel) da rea do funcionrio que se quer. Utilizar -1 caso se queira a rea imediata do funcionrio.
	 * - Por exemplo: o funcionrio est em uma rea de status 5 (Diviso) e se quer a rea superior de nvel 3 (Diretoria).
	 * @return int O ID da rea do funcionrio.
	 * @access public
	 */
	function getEmployeeAreaID($employeeID, $areaStatusID = -1)
	{
		$employee = $this->getEmployee($employeeID);
		if (!$employee)
			return false;

		/* requer a rea do funcionrio */
		if ($areaStatusID == -1)
			return $employee['area_id'];
		else
		{
			/* verifica se a rea do funcionrio j est no nvel solicitado */
			$currentArea = $this->getArea($employee['area_id']);
			if ($currentArea['area_status_id'] == $areaStatusID)
				return $currentArea['area_id'];
			else
				return $this->getParentAreaID($currentArea['area_id'], $areaStatusID);
		}
	}

	/**
	 * Busca o ID da localidade de um funcionrio.
	 *
	 * Este mtodo buscar o funcionrio pelo seu ID e retornar o ID da sua localidade.
	 * @param int $employeeID O ID do funcionrio.
	 * @return int O ID da localidade do funcionrio.
	 * @access public
	 */
	function getEmployeeLocalID($employeeID)
	{
		$employee = $this->getEmployee($employeeID);
		if (!$employee)
			return false;

		return $employee['localidade_id'];
	}

	/**
	 * Busca o ID do status de um funcionrio.
	 *
	 * Este mtodo buscar um funcionrio pelo seu ID e retornar o ID de status do funcionrio.
	 * Status de funcionrio corresponde  sua situao: ativo, desligado, etc.
	 * @param int $employeeID O ID do funcionrio.
	 * @return int O ID do status do funcionrio.
	 * @access public
	 */
	function getEmployeeStatusID($employeeID)
	{
		$employee = $this->getEmployee($employeeID);
		if (!$employee)
			return false;

		return $employee['funcionario_status_id'];
	}

	/**
	 * Busca o ID do centro de custo de um funcionrio.
	 *
	 * Este mtodo ir buscar um funcionrio pelo seu ID e retornar o ID do centro de custo do funcionrio.
	 * O centro de custo no  obrigatrio por funcionrio. Neste caso, se necessrio, busque o centro de custo da rea ou da localidade.
	 * @param int $employeeID O ID do funcionrio.
	 * @return int O ID do centro de custo do funcionrio.
	 * @access public
	 */
	function getEmployeeCostCenterID($employeeID)
	{
		$employee = $this->getEmployee($employeeID);
		if (!$employee)
			return false;

		return $employee['centro_custo_id'];
	}

	/**
	 * Busca o ID do cargo de um funcionrio.
	 *
	 * Este mtodo buscar um funcionrio pelo seu ID e retornar o ID do cargo do funcionrio.
	 * @param int $employeeID O ID do funcionrio.
	 * @return int O ID do cargo do funcionrio.
	 * @access public
	 */
	function getEmployeeJobTitleID($employeeID)
	{
		$employee = $this->getEmployee($employeeID);
		if (!$employee)
			return false;

		return $employee['cargo_id'];
	}

	/**
	 * Busca o ID da categoria de um funcionrio.
	 *
	 * Este mtodo buscar um funcionrio pelo seu ID e retornar o ID da categoria do funcionrio.
	 * A categoria corresponde ao tipo de vnculo do funcionrio com a organizacao. Por exemplo:
	 * - funcionrio
	 * - estagirio
	 * - terceirizado, etc
	 * @param int $employeeID O ID do funcionrio.
	 * @return int O ID do categoria do funcionrio.
	 * @access public
	 */
	function getEmployeeCategoryID($employeeID)
	{
		$employee = $this->getEmployee($employeeID);
		if (!$employee)
			return false;

		return $employee['funcionario_categoria_id'];
	}

	/**
	 * Busca o nvel de um funcionrio.
	 *
	 * Este mtodo buscar o funcionrio pelo seu ID e retornar o nvel do funcionrio dentro do cargo.
	 * Geralmente um cargo (por exemplo: auxiliar tecnico)  composto por nveis de evoluo na carreira: 1,2,3,...
	 * @param int $employeeID O ID do funcionrio.
	 * @return int A quantidade de nveis do funcionrio.
	 * @access public
	 */
	function getEmployeeLevel($employeeID)
	{
		$employee = $this->getEmployee($employeeID);
		if (!$employee)
			return false;

		if ($employee['nivel'] === null)
			return false;

		return $employee['nivel'];
	}

	/**
	 * Busca o ID do titular da rea de um funcionrio.
	 *
	 * Este mtodo buscar o titular da rea do funcionrio, e poder subir na hierarquia, buscando o titular de reas superiores  rea do funcionrio.
	 * @param int $employeeID O ID do funcionrio.
	 * @param int $areaStatusID O ID do status de rea (nvel) da rea do funcionrio que se quer o titular. Utilizar -1 caso se queira o titular da rea imediata do funcionrio.
	 * @return int O ID do titular da rea do funcionrio:
	 * - titular_funcionario_id
	 * @access public
	 */
	function getEmployeeSupervisorID($employeeID, $areaStatusID = -1)
	{
		return $this->getAreaSupervisorID($this->getEmployeeAreaID($employeeID, $areaStatusID));
	}

	/**
	 * Busca o ID do titular atual da rea do funcionrio.
	 *
	 * Caso haja um substituto, este ser o titular atual; caso contrrio, o titular atual  o prprio titular da rea.
	 * @param int $employeeID O ID do funcionrio.
	 * @param int $areaStatusID O ID do status de rea (nvel) da rea do funcionrio que se quer o titular atual. Utilizar -1 caso se queira o titular atual da rea imediata do funcionrio.
	 * @return int O ID do titular atual da rea do funcionrio:
	 * - titular_funcionrio_id ou
	 * - substituto_funcionario_id
	 * @access public
	 */
	function getEmployeeCurrentSupervisorID($employeeID, $areaStatusID = -1)
	{
		return $this->getAreaCurrentSupervisorID($this->getEmployeeAreaID($employeeID, $areaStatusID));
	}

	/**
	 * Busca o ID do responsvel administrativo atual de uma rea.
	 *
	 * A precedncia para definir quem  o responsvel administrativo : auxiliar administrativo, substituto e titular.
	 * @param int $employeeID O ID do funcionrio.
	 * @param int $areaStatusID O ID do status de rea (nvel) da rea do funcionrio que se quer o responsvel administrativo. Utilizar -1 caso se queira o responsvel administrativo da rea imediata do funcionrio.
	 * @return int O ID do responsvel administrativo atual da rea do funcionrio:
	 * - titular_funcionrio_id ou
	 * - substituto_funcionario_id ou
	 * - auxiliar_funcionario_id
	 * @access public
	 */
	function getEmployeeCurrentAdministrativeResponsibleID($employeeID, $areaStatusID = -1)
	{
		return $this->getAreaCurrentAdministrativeResponsibleID($this->getEmployeeAreaID($employeeID, $areaStatusID));
	}
}
?>
